##TRANSFORMERS

Un transformer est une classe qui va appliquer une transformation sur les données à insérer dans la base de données cible. Elles servent à couvrir les cas où une requête SQL ne suffit pas à formatter les données.

Un exemple d'utilisation est la transformation d'un nom de produit en Slug : rien ne permet, en SQL pûr, de faire cette transformation.

On implémente donc un transformer auquel on va passer le nom, qui va générer le Slug, puis retourner la donnée slugifiée avant l'écriture en base de données.

###Créer un transformer

La création d'un transformer se fait en implémentant l'interface `TransformerInterface`

Exemple :

```php
<?php

declare(strict_types=1);

namespace App\Service\Transformer;

use Behat\Transliterator\Transliterator;

class Slugify implements TransformerInterface
{

    public function getType(): string
    {
        return 'slugify';
    }

    public function execute($data)
    {
        $slug = Transliterator::transliterate(str_replace('\'', '-', trim($data)));
        return substr($slug, 0, 254);
    }
}
```

La valeur retournée par `getType` sera utilisée pour matcher la configuration `migration_mapping.yaml`. Pour utiliser ce transformer on utilisera la config suivante :

```yaml
migration_mapping:
  table:
    ...
    fields:
      ...
      slug:
        field: slug
        transformer: slugify
```

Les `$data` transmises sont résolues pour chaque ligne de données en récupérer la clé `field` dans la ligne traitée

###Transformers disponibles

####Slugify
**type**: `slugify`

Applique la fonction de transformation en slug sur une chaîne de caractères.

Note : dans Sylius, les slugs sont limités à 255 caractères. Le transformer Slugify tronque donc la chaîne de caractères à 255 caractères.

####SlugifyToUppercase
**type**: `slugify-uppercase`

Applique le transformer [Slugify](#slugify) et retourne le résultat en majuscules

####StringLimitLength
**type**: `string_limit_length`

Tronque une chaîne de caractères à 255 caractères.

####UrlToNewSite
**type**: `url-to-new-site`

Remplace les éléments dans la chaîne de caractères en entrée avec les éléments configurés (voir [la documentation à ce sujet](CONFIG.md)) pour générer une URL adaptée au nouveau site

####PathToSyliusImage
**type**: `path-to-sylius-image`

Remplace les éléments dans la chaîne de caractères en entrée avec les éléments configurés (voir [la documentation à ce sujet](CONFIG.md)) pour générer un chemin d'image adapté au nouveau site

####TransformToSyliusLocaleCode
**type**: `transform-to-sylius-locale-code`

Transforme un code de langue legacy ShopByDedi (FR, EN, ...) en code de locale Sylius (fr_FR, en_US, ...)
