<?php

declare(strict_types=1);

namespace Dedi\SyliusMigrationPlugin\Command;

use Dedi\SyliusMigrationPlugin\Service\MigrationExecutorInterface;
use Exception;
use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Input\InputOption;
use Symfony\Component\Console\Output\OutputInterface;

class MigrateCommand extends Command
{
    private MigrationExecutorInterface $migrationExecutor;

    public function __construct(
        string $name = null,
        MigrationExecutorInterface $migrator
    ) {
        parent::__construct($name);
        $this->migrationExecutor = $migrator;
    }

    protected function configure(): void
    {
        $this
            ->setDescription('Migrate database from a Legacy Database to Sylius')
            ->addOption('dry-run', null, InputOption::VALUE_NONE, 'Only execute a few lines per table and prints statement instead of executing them')
        ;
    }

    protected function execute(InputInterface $input, OutputInterface $output): int
    {
        /** @var bool $dryRun */
        $dryRun = $input->getOption('dry-run');

        if (true === $dryRun) {
            $output->writeln('<info>Running in dry-mode. No statement will be executed on databases</info>');
            $output->setVerbosity(OutputInterface::VERBOSITY_DEBUG);
        }

        try {
            $executionStartTime = microtime(true);
            $this->migrationExecutor->execute(!$dryRun);

            $executionEndTime = microtime(true);
            $seconds = round($executionEndTime - $executionStartTime);
            $hours = round($seconds / 3600);
            $mins = round($seconds / 60 % 60);
            $secs = round($seconds % 60);
            $output->writeln(sprintf('<info>Migration terminée en %02d:%02d:%02d</info>', $hours, $mins, $secs));

            return Command::SUCCESS;
        } catch (Exception $e) {
            $output->writeln(sprintf('<error>%s</error>', $e->getMessage()));
        }

        return Command::FAILURE;
    }
}
