<?php

declare(strict_types=1);

namespace Dedi\SyliusMigrationPlugin\Config;

use Dedi\SyliusMigrationPlugin\Definition\ColumnMappingDefinitionFactory;
use Dedi\SyliusMigrationPlugin\Definition\TableMappingDefinition;
use Dedi\SyliusMigrationPlugin\Dto\FilterMappingDefinition;
use Dedi\SyliusMigrationPlugin\Dto\OrderMappingDefinition;
use Dedi\SyliusMigrationPlugin\Dto\SearchMappingDefinition;
use Doctrine\Common\Collections\ArrayCollection;

class MigrationMappingConfig
{
    private array $tableDefinitions = [];

    private ColumnMappingDefinitionFactory $columnMappingDefinitionFactory;

    public function __construct(ColumnMappingDefinitionFactory $columnMappingDefinitionFactory)
    {
        $this->columnMappingDefinitionFactory = $columnMappingDefinitionFactory;
    }

    public function addTableMappingDefinition(
        string $sourceTableName,
        string $targetTableName,
        string $sql,
        string $sourcePrimaryKey,
        array $columnDefinitions,
        array $extra,
        array $search,
        array $filter,
        array $logginColumns,
        ?array $order = null
    ): void {
        $columnMappingDefinitions = new ArrayCollection();
        foreach ($columnDefinitions as $column => $definition) {
            $columnMappingDefinitions->add($this->columnMappingDefinitionFactory->createFromDefinition($column, $definition));
        }

        $searchMappingDefinitions = new ArrayCollection();
        foreach ($search as $definition) {
            $searchMappingDefinitions->add(SearchMappingDefinition::fromArray($definition));
        }

        $filterMappingDefinitions = new ArrayCollection();
        foreach ($filter as $definition) {
            $filterMappingDefinitions->add(FilterMappingDefinition::fromArray($definition));
        }

        $tableMappingDefinition = new TableMappingDefinition(
            $sourceTableName,
            $targetTableName,
            $sql,
            $sourcePrimaryKey,
            $columnMappingDefinitions->toArray(),
            $extra,
            $searchMappingDefinitions->toArray(),
            $filterMappingDefinitions->toArray(),
            $logginColumns,
            null !== $order ? OrderMappingDefinition::fromArray($order) : null
        );

        $this->tableDefinitions[] = $tableMappingDefinition;
    }

    /** @return iterable|TableMappingDefinition[] */
    public function getTableDefinitions(): iterable
    {
        return $this->tableDefinitions;
    }
}
