<?php

declare(strict_types=1);

namespace Dedi\SyliusMigrationPlugin\DependencyInjection;

use Symfony\Component\Config\Definition\Builder\TreeBuilder;
use Symfony\Component\Config\Definition\ConfigurationInterface;

final class Configuration implements ConfigurationInterface
{
    public function getConfigTreeBuilder(): TreeBuilder
    {
        $treeBuilder = new TreeBuilder('dedi_sylius_migration');
        $treeBuilder
            ->getRootNode()
            ->children()
                ->arrayNode('config')
                    ->children()
                        ->booleanNode('commit')
                            ->defaultTrue()
                        ->end()
                        ->integerNode('debug_limit_statements')
                            ->defaultValue(-1)
                        ->end()
                        ->booleanNode('generate_relative_path_for_images')
                            ->defaultTrue()
                        ->end()
                        ->scalarNode('legacy_site_url')
                            ->isRequired()
                        ->end()
                        ->scalarNode('legacy_site_media_path')
                            ->isRequired()
                        ->end()
                        ->scalarNode('new_site_media_path')
                            ->defaultValue('legacy')
                        ->end()
                        ->scalarNode('new_site_url')
                            ->defaultValue('')
                        ->end()
                    ->end()
                ->end()
            ->end()
        ;

        $treeBuilder = $this->getMigrationMappingConfig($treeBuilder);
        $treeBuilder = $this->getMigrationActionConfig($treeBuilder);

        return $treeBuilder;
    }

    private function getMigrationMappingConfig(TreeBuilder $treeBuilder): TreeBuilder
    {
        $rootNode = $treeBuilder
            ->getRootNode();

        $rootNode
            ->children()
                ->arrayNode('migration_mapping')
                ->useAttributeAsKey('name')
                ->arrayPrototype()
                    ->children()
                        ->scalarNode('table')->end()
                        ->scalarNode('sql')->end()
                        ->scalarNode('primary_key')->end()
                        ->arrayNode('order')
                            ->children()
                                ->scalarNode('column')->end()
                                ->scalarNode('sort')->end()
                            ->end()
                        ->end()
                        ->arrayNode('filter')
                            ->arrayPrototype()
                                ->children()
                                    ->scalarNode('filter_column')->end()
                                    ->scalarNode('value')->end()
                                    ->scalarNode('type')->end()
                                ->end()
                            ->end()
                        ->end()
                        ->arrayNode('fields')
                            ->useAttributeAsKey('name')
                            ->arrayPrototype()
                                ->beforeNormalization()
                                    ->ifString()->then(function ($v) { return ['field' => $v]; })
                                ->end()
                                ->children()
                                    ->scalarNode('field')->defaultNull()->end()
                                    ->scalarNode('sql')->defaultNull()->end()
                                    ->scalarNode('value')->defaultNull()->end()
                                    ->scalarNode('alias')->defaultNull()->end()
                                    ->scalarNode('expr')->defaultNull()->end()
                                    ->scalarNode('transformer')->defaultNull()->end()
                                ->end()
                            ->end()
                        ->end()
                        ->arrayNode('extra')
                            ->useAttributeAsKey('param')
                            ->arrayPrototype()
                                ->beforeNormalization()
                                    ->ifString()->then(function ($v) { return ['value' => $v]; })
                                ->end()
                                ->children()
                                    ->scalarNode('value')->end()
                                ->end()
                            ->end()
                        ->end()
                        ->arrayNode('search')
                            ->arrayPrototype()
                                ->children()
                                    ->scalarNode('search_column')->end()
                                    ->scalarNode('value_column')->end()
                                    ->scalarNode('type')->end()
                                ->end()
                            ->end()
                        ->end()
                        ->arrayNode('logging_columns')
                            ->scalarPrototype()->end()
                        ->end()
                    ->end()
                ->end()
            ->end()
        ;

        return $treeBuilder;
    }

    private function getMigrationActionConfig(TreeBuilder $treeBuilder): TreeBuilder
    {
        $rootNode = $treeBuilder
            ->getRootNode();

        $rootNode
            ->children()
            ->arrayNode('migration_actions')
                ->children()
                    ->arrayNode('truncate_table_before_migration')
                        ->scalarPrototype()->end()
                    ->end()
                    ->arrayNode('execute_query_before_migration')
                        ->scalarPrototype()->end()
                    ->end()
                    ->arrayNode('clean_extra_legacy_column_after_migration')
                        ->beforeNormalization()->castToArray()->end()
                        ->scalarPrototype()->end()
                    ->end()
                ->end()
            ->end();

        return $treeBuilder;
    }
}
