<?php

declare(strict_types=1);

namespace Dedi\SyliusMigrationPlugin\Service\MigrationAction;

use Dedi\SyliusMigrationPlugin\Config\MigrationMappingConfig;
use Dedi\SyliusMigrationPlugin\Service\Logger\MigrationLoggerInterface;
use Doctrine\DBAL\Connection;
use Psr\Log\LogLevel;

/**
 * Exemple de config :
 *
 *   truncate_table_before_migration:
 *     execute: true|false
 */
class CleanExtraLegacyColumnAfterMigration extends AbstractMigrationAction implements PostMigrationActionInterface
{
    private Connection $connection;
    private MigrationMappingConfig $migrationMappingConfig;
    private MigrationLoggerInterface $migrationLogger;

    public function __construct(
        Connection $connection,
        MigrationMappingConfig $migrationMappingConfig,
        MigrationLoggerInterface $migrationLogger
    ) {
        $this->connection = $connection;
        $this->migrationMappingConfig = $migrationMappingConfig;
        $this->migrationLogger = $migrationLogger;
    }

    public function getType(): string
    {
        return 'clean_extra_legacy_column_after_migration';
    }

    public function execute(bool $commit): void
    {
        if (1 !== count($this->options) && true !== $this->options[0]) {
            return;
        }

        foreach ($this->migrationMappingConfig->getTableDefinitions() as $tableDefinition) {
            if (0 === count($tableDefinition->getExtra()) || !array_key_exists('legacy_column_name', $tableDefinition->getExtra())) {
                continue;
            }

            $sql = sprintf('ALTER TABLE %s DROP COLUMN %s', $tableDefinition->getTargetTableName(), $tableDefinition->getExtra()['legacy_column_name']['value']);

            $this->migrationLogger->log(LogLevel::INFO, $sql);
            if ($commit) {
                $this->connection->executeQuery($sql);
            }
        }
    }
}
