<?php

declare(strict_types=1);

namespace Dedi\SyliusMigrationPlugin\Service\MigrationAction;

use Dedi\SyliusMigrationPlugin\Service\Logger\MigrationLoggerInterface;
use Doctrine\DBAL\Connection;
use Psr\Log\LogLevel;

/**
 * Exemple de config :
 *
 *   truncate_table_before_migration:
 *     tables:
 *       - [nom_de_la_table]
 *       - ...
 */
class TruncateTableBeforeMigration extends AbstractMigrationAction implements PreMigrationActionInterface
{
    private Connection $connection;
    private MigrationLoggerInterface $logger;

    public function __construct(Connection $connection, MigrationLoggerInterface $logger)
    {
        $this->connection = $connection;
        $this->logger = $logger;
    }

    public function getType(): string
    {
        return 'truncate_table_before_migration';
    }

    public function execute(bool $commit): void
    {
        foreach ($this->options as $table) {
            $sql = sprintf('DELETE FROM `%s`', $table);
            $this->logger->log(LogLevel::INFO, $sql);

            if ($commit) {
                $this->connection->executeQuery($sql);
            }
        }
    }
}
